rm(list = ls());
##### Transformed linear regression (TLR) model for mileage prediction #####

##### Description #####
# This code estimates a TLR model that predicts the annual motorized mileage of a household 

# Read data for estimating mileage prediction model
data1 <<- read.table("C:/workspace/VFC_Package_Beta/Model_Estimation/az_hhld_mileage_estimation_data.csv", header=T, sep=",");

##### Variable Definitions ######
#	DRVRCNT		    - Number of drivers in household
#	NUMADLT		    - Count of adult household members at least 18 years old
#	RUR		        - Household resides in rural area (from variable URBRUR)
#	INCOME1		    - Lowest income household (< $25,000)
#	INCOME5		    - Highest income household (>= $100,000)
#	NUMCHILD	    -	Number of children in the household
#	WORK0		      - Zero worker household
#	WORK2		      - Two worker household
#	HSIZE4		    - Household size = 4 or more
#	SF_Own		    - Single family housing unit (owned)
#	Retired		    - Retired household (one/two person) with no children
#	Prop_HH_INC5	-	Proportion of households in the highest income quintile (INC5/TOTAL_HH)
#	Prop_SFHH		  - Proportion of single family housing units in the TAZ
#	AUTO10_Q1		  - TAZ with high regional employment accessible within 10 minutes by auto (1st Quartile)
# Trans_motor   - Transformed motorized mileage (dependant variable)
#################################

# Model Estimation
mile_model <- lm(Trans_motor ~ DRVRCNT + NUMADLT + RUR + INCOME1  +INCOME5 + NUMCHILD + WORK0 + WORK2 + HSIZE4 + SF_Own + Retired + Prop_HH_INC5 + Prop_SFHH + AUTO10_Q1, 
                 data = data1);


# Model results
summary(mile_model);

####################################################### End of Code ###################################################################################