##### Multinomial Logit (MNL) model for number of alternatives #####

##### Description #####
# This code estimates and applies an MNL model of number of alternatives on the same data set 

library("utils")   # to use csv read function
library("foreign") # to use spss read function
library("pastecs") # to use descriptive statistics function
library("mlogit")
library("graphics")
library("VGAM")
library("ZeligChoice") # This package is for actual mlogit function
library("aod")
library("plotrix")

##### Read the data #####
pst <- proc.time()

data <- read.table("C:/workspace/VFC_Package_Beta/Model_Estimation/num_alt_r.dat", 
                   col.names = c("HOUSEID",
                                 "TAZ",
                                 "RAZ",
                                 "uno",
                                 "sero",
                                 "id",
                                 "CAR1",
                                 "CAR2",
                                 "CAR3",
                                 "VAN1",
                                 "VAN2",
                                 "VAN3",
                                 "SUV1",
                                 "SUV2",
                                 "SUV3",
                                 "PICK1",
                                 "PICK2",
                                 "PICK3",
                                 "MOTOR",
                                 "DCAR1",
                                 "DCAR2",
                                 "DCAR3",
                                 "DVAN1",
                                 "DVAN2",
                                 "DVAN3",
                                 "DSUV1",
                                 "DSUV2",
                                 "DSUV3",
                                 "DPICK1",
                                 "DPICK2",
                                 "DPICK3",
                                 "DMOTER",
                                 "PUMA",
                                 "DISTRICT",
                                 "DRVRCNT",
                                 "HH_HISP",
                                 "HH_RACE",
                                 "HHBG",
                                 "HHC_MSA",
                                 "HHFAMINC",
                                 "HHMETDIV",
                                 "HHRELATD",
                                 "HHRESP",
                                 "HHSIZE",
                                 "HHVEHCNT",
                                 "HOMEGEO",
                                 "HOMELAT",
                                 "HOMELONG",
                                 "HOMEOWN",
                                 "HOMETYPE",
                                 "IMPTHOWN",
                                 "NUMADLT",
                                 "RAIL",
                                 "RESP_CNT",
                                 "TRAVDAY",
                                 "URBAN",
                                 "URBANSIZE",
                                 "URBRUR",
                                 "WRKCOUNT",
                                 "HH13WGT",
                                 "WTHHFIN",
                                 "HHR_HISP",
                                 "HHR_RACE",
                                 "LIF_CYC",
                                 "CNTTDHH",
                                 "num_pers",
                                 "num_trips",
                                 "TRPCNT",
                                 "VEHCNT",
                                 "YEAR",
                                 "POP1",
                                 "POP2",
                                 "POP3",
                                 "POP4",
                                 "HH1",
                                 "HH2",
                                 "HH3",
                                 "HH4",
                                 "EMP1",
                                 "EMP2",
                                 "EMP3",
                                 "EMP4",
                                 "EMP5",
                                 "INC1",
                                 "INC2",
                                 "INC3",
                                 "INC4",
                                 "INC5",
                                 "TOTAREA",
                                 "OFFAREA",
                                 "ENROLL",
                                 "RFLAG",
                                 "APASS",
                                 "DU9",
                                 "DU19",
                                 "DU29",
                                 "DU30",
                                 "SFHH",
                                 "MFHH",
                                 "POPCORR",
                                 "POPINST",
                                 "POPMINST",
                                 "WAHEMP",
                                 "CONSTEMP",
                                 "PUBOFF",
                                 "PUBPUB",
                                 "ASUENROLL",
                                 "K8",
                                 "G9TO12",
                                 "NSBEMP",
                                 "POPDEN",
                                 "EMPDEN",
                                 "OWN",
                                 "URB",
                                 "RUR",
                                 "SF",
                                 "MF",
                                 "INCOME1",
                                 "INCOME2",
                                 "INCOME3",
                                 "INCOME4",
                                 "INCOME5",
                                 "NUMCHILD",
                                 "child_pres",
                                 "WORK0",
                                 "WORK1",
                                 "WORK2",
                                 "WORK3",
                                 "HSIZE1",
                                 "HSIZE2",
                                 "HSIZE3",
                                 "HSIZE4",
                                 "annual_bike_mile_sum",
                                 "annual_walk_mile_sum",
                                 "trp_bike_walk_ind",
                                 "per_bike_ind",
                                 "per_walk_ind",
                                 "ann_mile_ind",
                                 "annual_non_motorized_mileage",
                                 "SF_Own",
                                 "Retired",
                                 "popden_per_100",
                                 "empden_per_100",
                                 "TOT_POP",
                                 "TOT_EMP",
                                 "TOTAL_HH",
                                 "Prop_HH_INC5",
                                 "Prop_HH_INC1",
                                 "Prop_SFHH",
                                 "Prop_MFHH",
                                 "annual_bw_trp_miles_hhld_PB",
                                 "annual_bw_trp_miles_hhld_raw",
                                 "trip_bike_walk_ind",
                                 "per_bike_walk_ind",
                                 "ind_non_motor_mileage",
                                 "annual_non_motor_mileage_new",
                                 "test",
                                 "TRANSIT20",
                                 "TRANSIT50",
                                 "AUTO20",
                                 "AUTO50",
                                 "RATE_TRANSIT20",
                                 "RATE_TRANSIT50",
                                 "RATE_AUTO20",
                                 "RATE_AUTO50",
                                 "CURRENT",
                                 "FUTURE",
                                 "Dev_area",
                                 "TAZ_density",
                                 "TRANSIT10",
                                 "TRANSIT30",
                                 "AUTO10",
                                 "AUTO30",
                                 "RATE_TRANSIT10",
                                 "RATE_TRANSIT30",
                                 "RATE_AUTO10",
                                 "RATE_AUTO30",
                                 "TAZ_den_Q4",
                                 "TAZ_den_Q3",
                                 "TAZ_den_Q2",
                                 "TAZ_den_Q1",
                                 "AUTO10_Q4",
                                 "AUTO10_Q3",
                                 "AUTO10_Q2",
                                 "AUTO10_Q1",
                                 "AUTO30_Q4",
                                 "AUTO30_Q3",
                                 "AUTO30_Q2",
                                 "AUTO30_Q1",
                                 "CAR0_3",
                                 "CAR4_9",
                                 "CAR_10",
                                 "VAN0_3",
                                 "VAN4_9",
                                 "VAN_10",
                                 "SUV0_3",
                                 "SUV4_9",
                                 "SUV_10",
                                 "PICK0_3",
                                 "PICK4_9",
                                 "PICK_10",
                                 "TAZ_den_Q4_Old",
                                 "TAZ_den_Q3_Old",
                                 "TAZ_den_Q2_Old",
                                 "TAZ_den_Q1_Old",
                                 "AUTO10_Q4_Old",
                                 "AUTO10_Q3_Old",
                                 "AUTO10_Q2_Old",
                                 "AUTO10_Q1_Old",
                                 "AUTO30_Q4_Old",
                                 "AUTO30_Q3_Old",
                                 "AUTO30_Q2_Old",
                                 "AUTO30_Q1_Old",
                                 "carmile",
                                 "vanmile",
                                 "suvmile",
                                 "pickmile",
                                 "NCAR",
                                 "NVAN",
                                 "NSUV",
                                 "NPICK",
                                 "YCAR0_5",
                                 "YCAR6_11",
                                 "YCAR12",
                                 "YVAN0_5",
                                 "YVAN6_11",
                                 "YVAN12",
                                 "YSUV0_5",
                                 "YSUV6_11",
                                 "YSUV12",
                                 "YPICK0_5",
                                 "YPICK6_11",
                                 "YPICK12",
                                 "TYPE0",
                                 "TYPE1",
                                 "TYPE2",
                                 "TYPE3",
                                 "TYPE4",
                                 "YMotor",
                                 "nveh_hhld",
                                 "Index1",
                                 "choice"),na.strings="-999")

##### Descriptive Statistics #####
# print(stat.desc(data,basic=TRUE, desc=TRUE, norm=FALSE, p=0.95))


data$lif_cyc_1 <- 0
data$lif_cyc_1[data$LIF_CYC == 4] <- 1
data$lif_cyc_2 <- 0
data$lif_cyc_2[data$LIF_CYC == 8] <- 1

##### Coefficient definitions for MNL model#####
data$const <- 1

data$inc1_1 <- data$INCOME1 * data$const 
data$inc2_1 <- data$INCOME2 * data$const
data$own_1 <- data$OWN * data$const
data$hsiz1_1 <- data$HSIZE1 * data$const
data$wrkr0_1 <- data$WORK0 * data$const
data$popden_1 <- data$POPDEN * data$const

data$inc1_2 <- data$INCOME1 * data$const
data$inc2_2 <- data$INCOME2 * data$const
data$hsiz1_2 <- data$HSIZE1 * data$const
data$promf_2 <- data$Prop_MFHH * data$const
data$wrkr2_1 <- data$WORK2 * data$const

data$lif_cyc4_3 <-  data$lif_cyc_1 * data$const
data$inc3_3 <- data$INCOME3 * data$const
data$wrkr2_3 <- data$WORK2 * data$const
data$INC1_3 <- data$INC1 * data$const

data$own_4 <- data$OWN * data$const
data$nadlt_4 <- data$NUMADLT * data$const
data$wrkr3_4 <- data$WORK3 * data$const
data$popden_4 <- data$POPDEN * data$const
data$chldpr_4 <- data$child_pres * data$const
data$lif_cyc8_4 <-  data$lif_cyc_2 * data$const

data$inc5_5 <- data$INCOME5 * data$const
data$lif_cyc8_5 <-  data$lif_cyc_2 * data$const
data$hsiz4_5 <- data$HSIZE4 * data$const
data$wrkr2_5 <- data$WORK2 * data$const

#### Multinomial Logit Model Estimation #####

number_of_alternatives <- zelig(list(id(Index1, 1) ~ inc1_1 + inc2_1 + own_1 + hsiz1_1 + wrkr0_1 + popden_1,
                                 id(Index1, 2) ~ inc1_2 + inc2_2 + hsiz1_2 + promf_2 + wrkr2_1, 
                                 id(Index1, 3) ~ lif_cyc4_3 + inc3_3 + wrkr2_3 + INC1_3,
                                 id(Index1, 4) ~ own_4 + nadlt_4 + wrkr3_4 + popden_4 + chldpr_4 + lif_cyc8_4,
                                 id(Index1, 5) ~ inc5_5 + lif_cyc8_5 + hsiz4_5 + wrkr2_5), model = "mlogit", data = data)
summary(number_of_alternatives)


######################################### Application ###############################################
num_alt <- 6
coefs <-coef(number_of_alternatives)
cons <- coefs[1:num_alt-1]
one_coef <- coefs[6:11]
two_coef <- coefs[12:16]
three_coef <- coefs[17:20]
four_coef <- coefs[21:26]
five_coef <- coefs[27:30]
six_coef <- data$INCOME1*0


### Utility computation
util_one_veh <- exp(cons[1] + data$INCOME1*one_coef[1] + data$INCOME2*one_coef[2] + data$OWN*one_coef[3]+ data$HSIZE1*one_coef[4] + data$WORK0*one_coef[5]
                + data$POPDEN*one_coef[6])
util_two_veh <- exp(cons[2] + data$INCOME1*two_coef[1] + data$INCOME2*two_coef[2] + data$HSIZE1*two_coef[3]+ data$Prop_MFHH*two_coef[4] + data$WORK2*two_coef[5])
util_three_veh <- exp(cons[3] + data$lif_cyc_1*three_coef[1] + data$INCOME3*three_coef[2] + data$WORK2*three_coef[3]+ data$INC1*three_coef[4])
util_four_veh <- exp(cons[4] + data$OWN*four_coef[1] + data$NUMADLT*four_coef[2] + data$WORK3*four_coef[3]+ data$POPDEN*four_coef[4]
                 +  data$child_pres*four_coef[5] +  data$lif_cyc_2*four_coef[6])
util_five_veh <- exp(cons[5] + data$INCOME5*five_coef[1] + data$lif_cyc_2*five_coef[2] + data$HSIZE4*five_coef[3] + data$WORK2*five_coef[4])
util_six_veh <- exp(six_coef)


### Probability computation
total_util <- util_one_veh + util_two_veh + util_three_veh + util_four_veh + util_five_veh + util_six_veh 
prob_one_veh <- (util_one_veh/total_util)
prob_two_veh <- (util_two_veh/total_util)
prob_three_veh <- (util_three_veh/total_util)
prob_four_veh <- (util_four_veh/total_util)
prob_five_veh <- (util_five_veh/total_util)
prob_six_veh <- (util_six_veh/total_util)


### Cumulative probability computation
cum_prob_one_veh <- prob_one_veh 
cum_prob_two_veh <- cum_prob_one_veh +  prob_two_veh 
cum_prob_three_veh <- cum_prob_two_veh +  prob_three_veh 
cum_prob_four_veh <- cum_prob_three_veh +  prob_four_veh 
cum_prob_five_veh <- cum_prob_four_veh +  prob_five_veh 
cum_prob_six_veh <- cum_prob_five_veh +  prob_six_veh 


### Predicted choices
rnum <- runif(4262);
rnum[rnum < cum_prob_one_veh] <- 2;
rnum[rnum < cum_prob_two_veh] <- 3;
rnum[rnum < cum_prob_three_veh] <- 4;
rnum[rnum < cum_prob_four_veh] <- 5;
rnum[rnum < cum_prob_five_veh] <- 6;
rnum[rnum <= 1] <- 7;

data$pred_own <- rnum - 2;

data$num_obs <- ifelse(data$nveh_hhld > 5, 5, data$nveh_hhld)
comp_obs_pred <- list(data$num_obs, data$pred_own)

### Observed vs. Predicted comparison chart
multhist(comp_obs_pred, xlab="Number of Vehicle Alternatives", ylab="Percent of Households", col = c("blue", "red"),  main = "Number of Vehicle Alternatives",
         legend.text = c("Observed", "Predicted"), names.arg = c("0", "1", "2", "3", "4", ">=5"), breaks=seq(-0.5,6, by = 1),  ylim=c(0,0.5), freq = FALSE) 

print(proc.time() - pst)
####################################################### End of code ###################################################################################